"""Create mesh and time boundary for the Okushiri island validation
"""


from Numeric import array, zeros, Float, allclose

from anuga.pmesh.mesh import *
from anuga.pmesh.mesh_interface import create_mesh_from_regions
from anuga.coordinate_transforms.geo_reference import Geo_reference

import project


def prepare_timeboundary(filename):
    """Converting benchmark 2 time series to NetCDF tms file.
    This is a 'throw-away' code taylor made for files like
    'Benchmark_2_input.txt' from the LWRU2 benchmark
    """

    textversion = filename[:-4] + '.txt'

    print 'Preparing time boundary from %s' %textversion
    from Numeric import array

    fid = open(textversion)

    #Skip first line
    line = fid.readline()

    #Read remaining lines
    lines = fid.readlines()
    fid.close()


    N = len(lines)
    T = zeros(N, Float)  #Time
    Q = zeros(N, Float)  #Values

    for i, line in enumerate(lines):
        fields = line.split()

        T[i] = float(fields[0])
        Q[i] = float(fields[1])


    #Create tms file
    from Scientific.IO.NetCDF import NetCDFFile

    print 'Writing to', filename
    fid = NetCDFFile(filename, 'w')

    fid.institution = 'Geoscience Australia'
    fid.description = 'Input wave for Benchmark 2'
    fid.starttime = 0.0
    fid.createDimension('number_of_timesteps', len(T))
    fid.createVariable('time', Float, ('number_of_timesteps',))
    fid.variables['time'][:] = T

    fid.createVariable('stage', Float, ('number_of_timesteps',))
    fid.variables['stage'][:] = Q[:]

    fid.createVariable('xmomentum', Float, ('number_of_timesteps',))
    fid.variables['xmomentum'][:] = 0.0

    fid.createVariable('ymomentum', Float, ('number_of_timesteps',))
    fid.variables['ymomentum'][:] = 0.0

    fid.close()


#-------------------------------------------------------------
if __name__ == "__main__":


    #Preparing points
    #(Only when using original Benchmark_2_Bathymetry.txt file)
    # FIXME: Replace by using geospatial data
    #
    #from anuga.abstract_2d_finite_volumes.data_manager import xya2pts
    #xya2pts(project.bathymetry_filename, verbose = True,
    #        z_func = lambda z: -z)


    # Prepare time boundary
    prepare_timeboundary(project.boundary_filename)


    #--------------------------------------------------------------------------
    # Create the triangular mesh based on overall clipping polygon with a
    # tagged
    # boundary and interior regions defined in project.py along with
    # resolutions (maximal area of per triangle) for each polygon
    #--------------------------------------------------------------------------

    base_resolution = 1

    # Basic geometry
    xleft   = 0
    xright  = 5.448
    ybottom = 0
    ytop    = 3.402

    # Outline 
    point_sw = [xleft, ybottom]
    point_se = [xright, ybottom]
    point_nw = [xleft, ytop]    
    point_ne = [xright, ytop]

    # Midway points (left)
    point_mtop = [xleft + (xright-xleft)/3+1, ytop]
    point_mbottom = [xleft + (xright-xleft)/3+1, ybottom]

    #Localised refined area for gulleys
    xl = 4.8
    xr = 5.3
    yb = 1.6
    yt = 2.3
    p0 = [xl, yb]
    p1 = [xr, yb]
    p2 = [xr, yt]
    p3 = [xl, yt]
    gulleys = [p0, p1, p2, p3]

    #Island area
    island_0 = [xleft + 2*(xright-xleft)/3+1.2, ytop-0.5]
    island_1 = [xleft + 2*(xright-xleft)/3+0.5, ybottom + 2*(ytop-ybottom)/3]
    island_2 = [xleft + (xright-xleft)/2+0.3, ybottom + 2*(ytop-ybottom)/3-0.3]
    island_3 = [xleft + (xright-xleft)/2+0.3, ybottom + (ytop-ybottom)/3+0.3]
    island_4 = [xleft + 2*(xright-xleft)/3+0.4, ybottom + (ytop-ybottom)/3-0.3]
    island_5 = [xleft + 2*(xright-xleft)/3+1.2, ybottom+0.2]
    island_6 = [xl-.01, yb]  #OK
    island_7 = [xl-.01, yt]  #OK      

    island = [island_0, island_1, island_2,
              island_3, island_4, island_5,
              island_6, island_7]


    

    bounding_polygon = [point_se,
                        point_ne,
                        point_mtop,
                        point_nw,
                        point_sw,
                        point_mbottom]
    

    interior_regions = [[gulleys, 0.00002*base_resolution],
                        [island, 0.00007*base_resolution]]
    


    

    print 'start create mesh from regions'

    meshname = project.mesh_filename + '.msh'
    m = create_mesh_from_regions(bounding_polygon,
                                 boundary_tags={'wall': [0, 1, 2, 4, 5],
                                                'wave': [3]},
                                 maximum_triangle_area=0.1*base_resolution,
                                 interior_regions=interior_regions,
                                 filename=project.mesh_filename,
                                 use_cache=True,
                                 verbose=True)

